@@ -15,20 +15,20 @@ from typing import TypeVar
 from typing import Union
 
 import gym
-import nptyping
 import numpy as np
 import torch
 from gym import Wrapper
 from gym import spaces
 from gym.spaces import Box
 from loguru import logger
-from nptyping import NDArray
 from numpy.typing import DTypeLike
+from numpy.typing import NDArray
 from torch.nn import functional as F
 from torchvision import transforms
+from tree import map_structure
 
 # All wrappers should operate only on ndarrays, no torch Tensors!
-ArrayType = nptyping.NDArray
+ArrayType = np.typing.NDArray
 DictActionType = dict[str, ArrayType]
 DictObservationType = dict[str, ArrayType]
 
@@ -44,7 +44,7 @@ class DictObsActionWrapper(Wrapper[DictObservationType, DictActionType]):
         env: gym.Env[Union[DictObservationType, NDArray], Union[DictObservationType, NDArray]],
         obs_key: str = "wrapped",
         action_key: str = "wrapped",
-    ):
+    ) -> None:
         super().__init__(env)
 
         # Build observation space
@@ -80,7 +80,7 @@ class DictObsActionWrapper(Wrapper[DictObservationType, DictActionType]):
 
     def observation(self, observation: Union[NDArray, DictObservationType]) -> DictObservationType:
         if self.wrapped_obs:
-            assert isinstance(observation, NDArray)
+            assert isinstance(observation, np.ndarray)
             return {self.obs_key: observation}
         else:
             return observation
@@ -89,7 +89,7 @@ class DictObsActionWrapper(Wrapper[DictObservationType, DictActionType]):
 class PixelObsWrapper(gym.ObservationWrapper):
     """Render state-based envs to pixels and use that as the observation."""
 
-    def __init__(self, env: gym.Env):
+    def __init__(self, env: gym.Env) -> None:
         super().__init__(env)
         pixels: NDArray = self.env.render(mode="rgb_array")
         # Some envs (eg CartPole-v1) won't render without being reset first.
@@ -105,7 +105,7 @@ class PixelObsWrapper(gym.ObservationWrapper):
 
 
 class ImageTransformWrapper(gym.ObservationWrapper):
-    def __init__(self, env: gym.Env, key: str, resolution: Optional[int] = None, greyscale: bool = False):
+    def __init__(self, env: gym.Env, key: str, resolution: Optional[int] = None, greyscale: bool = False) -> None:
         """Transform a uint8 image into the format expected by a model.
 
         We actually leave images as uint8s until training, for the 4x lower data size."""
@@ -156,7 +156,7 @@ class ImageTransformWrapper(gym.ObservationWrapper):
 class ClipActionWrapper(gym.ActionWrapper):
     """Clip actions in Box envs that are out of bounds."""
 
-    def __init__(self, env: gym.Env):
+    def __init__(self, env: gym.Env) -> None:
         assert isinstance(env.action_space, gym.spaces.Dict)
         super().__init__(env)
 
@@ -176,7 +176,7 @@ class DiscreteActionToIntWrapper(gym.ActionWrapper):
 
     So for now I'll leave them in floats in my code, and convert them in this wrapper."""
 
-    def __init__(self, env: gym.Env):
+    def __init__(self, env: gym.Env) -> None:
         super().__init__(env)
 
     def action(self, action: np.floating) -> Union[np.floating, int]:
@@ -194,15 +194,25 @@ class OneHotDiscrete(gym.spaces.Discrete):
 
 
 class OneHotMultiDiscrete(gym.spaces.MultiDiscrete):
-    def __init__(self, nvec: List[int], dtype: DTypeLike = np.int64):
-        assert np.issubdtype(dtype, (int, np.integer))
+    def __init__(self, nvec: List[int], dtype: DTypeLike = np.float32) -> None:
+        # the dtype is default float32 since that's what torch.OneHotCategorical.sample() returns,
+        # for some reason (presumably to make backprop work).
+        # This is kinda weird tho because then it makes nvec a float32 too.. this superclass really expects an int
+        # But we need self.dtype to be the type that we pass around during rollouts,
+        # since this is where it gets the dtype to use for storing the actions.
+        # One option is to make self.dtype be an integer type, use integers all thru rollouts/inference,
+        # and floats in training. Confusing though.
+        # Another is to always use float, and just patch this class to make it handle that properly.
+        # That's the approach I've gone with for now.
         self.max_categories = max(nvec)
         super().__init__(nvec=nvec, dtype=dtype)
 
     def sample(self, mask: Optional[NDArray] = None) -> NDArray:
         assert mask is None
-        sample = super().sample()
-        return F.one_hot(torch.from_numpy(sample), num_classes=self.max_categories).numpy()
+        sample = (self.np_random.random(self.nvec.shape) * self.nvec).astype(np.int32)
+        return (
+            F.one_hot(torch.from_numpy(sample).long(), num_classes=self.max_categories).to(dtype=torch.float32).numpy()
+        )
 
 
 class OneHotActionWrapper(gym.ActionWrapper):
@@ -214,7 +224,7 @@ class OneHotActionWrapper(gym.ActionWrapper):
     for each dimension. Since we'll use max(num_classes) as the one-hot vector length for all.
     """
 
-    def __init__(self, env: gym.Env):
+    def __init__(self, env: gym.Env) -> None:
         super().__init__(env)
         assert isinstance(env.action_space, gym.spaces.Dict)
         self._env = env
@@ -234,7 +244,7 @@ class OneHotActionWrapper(gym.ActionWrapper):
         out: dict[str, Any] = copy.copy(action_dict)
         for k, space in self._env.action_space.spaces.items():
             action = action_dict[k]
-            assert isinstance(action, NDArray)
+            assert isinstance(action, np.ndarray)
 
             if isinstance(space, gym.spaces.Discrete):
                 assert action.shape == (1, space.n)
@@ -252,7 +262,7 @@ class OneHotActionWrapper(gym.ActionWrapper):
 
 
 class TimeLimit(gym.Wrapper[GenericObservationType, GenericActionType]):
-    def __init__(self, env: gym.Env, max_episode_steps: Optional[int] = None):
+    def __init__(self, env: gym.Env, max_episode_steps: Optional[int] = None) -> None:
         super().__init__(env)
         self._max_episode_steps: Optional[int] = max_episode_steps
         self._elapsed_steps: Optional[int] = None
@@ -273,7 +283,7 @@ class TimeLimit(gym.Wrapper[GenericObservationType, GenericActionType]):
 
 
 class ElapsedTimeWrapper(gym.Wrapper[DictObservationType, GenericActionType]):
-    def __init__(self, env: gym.Env, max_episode_steps: Optional[int] = None):
+    def __init__(self, env: gym.Env, max_episode_steps: Optional[int] = None) -> None:
         super().__init__(env)
         self._max_episode_steps: Optional[int] = max_episode_steps
         self._elapsed_steps: Optional[int] = None
@@ -305,7 +315,7 @@ class NormalizeActions(gym.ActionWrapper):
 
     Does not adapt non-Box spaces or dimensions that have non-finite bounds."""
 
-    def __init__(self, env: gym.Env):
+    def __init__(self, env: gym.Env) -> None:
         super().__init__(env)
         self._env = env
         self.bounds = {}
@@ -337,7 +347,7 @@ class NormalizeActions(gym.ActionWrapper):
 
 
 class ScaleRewards(gym.RewardWrapper):
-    def __init__(self, env: gym.Env, scale: float = 1.0, func: Callable[[float], float] = lambda x: x):
+    def __init__(self, env: gym.Env, scale: float = 1.0, func: Callable[[float], float] = lambda x: x) -> None:
         super().__init__(env)
         self.scale = scale
         self.func = func
@@ -347,7 +357,7 @@ class ScaleRewards(gym.RewardWrapper):
 
 
 class ActionRepeat(gym.Wrapper[GenericObservationType, GenericActionType]):
-    def __init__(self, env: gym.Env, amount: int):
+    def __init__(self, env: gym.Env, amount: int) -> None:
         super().__init__(env)
         self._amount = amount
 
@@ -359,6 +369,7 @@ class ActionRepeat(gym.Wrapper[GenericObservationType, GenericActionType]):
             obs, reward, done, info = self.env.step(action)
             total_reward += reward
             current_step += 1
+        # Note: this will just return the info of the most recent step
         return obs, total_reward, done, info
 
 
@@ -370,7 +381,7 @@ class DictFrameStack(gym.ObservationWrapper):
     the observation at t=0 will be repeated back in time to fill.
     """
 
-    def __init__(self, env: gym.Env, num_stack: int):
+    def __init__(self, env: gym.Env, num_stack: int) -> None:
         super().__init__(env)
         self._env = env
         self.num_stack = num_stack
@@ -407,3 +418,56 @@ class DictFrameStack(gym.ObservationWrapper):
             # images should be chw, and scalars/vectors should be stacked on dim 0 also
             out[k] = np.concatenate(self.history[k], axis=0)
         return out
+
+
+class Torchify(gym.Wrapper):
+    """Allow the client to use Torch tensors instead of the usual numpy arrays."""
+
+    def to_torch(self, x):
+        return map_structure(lambda x: torch.from_numpy(x) if isinstance(x, np.ndarray) else x, x)
+
+    def to_numpy(self, x):
+        return map_structure(lambda x: x.numpy() if isinstance(x, torch.Tensor) else x, x)
+
+    def step(self, action):
+        action = self.to_numpy(action)
+        obs, reward, done, info = self.env.step(action)
+        return self.to_torch(obs), float(reward), bool(done), self.to_torch(info)
+
+    def reset(self, **kwargs):
+        obs = self.env.reset(**kwargs)
+        obs = self.to_torch(obs)
+        return obs
+
+
+class RecordEpisodeStatistics(gym.Wrapper):
+    """This wrapper will keep track of cumulative rewards and episode lengths.
+
+    At the end of an episode, the statistics of the episode will be added to ``info``
+
+    Frame skipping is currently **NOT** taken into account.
+
+    I use this instead of gym.RecordEpisodeStatistics because that has a nested info key that I didn't
+    want to deal with supporting.
+    And I use this wrapper instead of building it into the worker, because we often want to record the reward
+    before any reward scaling is applied.
+    """
+
+    def __init__(self, env: gym.Env) -> None:
+        super().__init__(env)
+        self.episode_return: float = 0
+        self.episode_length: int = 0
+
+    def reset(self, **kwargs):
+        observations = super().reset(**kwargs)
+        self.episode_return = 0
+        self.episode_length = 0
+        return observations
+
+    def step(self, action):
+        obs, reward, done, info = self.env.step(action)
+        self.episode_return += reward
+        self.episode_length += 1
+        info["cumulative_episode_return"] = self.episode_return
+        info["cumulative_episode_length"] = self.episode_length
+        return obs, reward, done, info
