@@ -44,6 +44,7 @@ from avalon.datagen.world_creation.constants import TRAIN_TASK_GROUPS
 from avalon.datagen.world_creation.constants import AvalonTask
 from avalon.datagen.world_creation.constants import AvalonTaskGroup
 from avalon.datagen.world_creation.constants import get_all_tasks_for_task_groups
+from avalon.datagen.world_creation.constants import int_to_avalon_task
 from avalon.datagen.world_creation.entities.spawn_point import PLAYER_SPAWN_POINT
 from avalon.datagen.world_creation.world_generator import AvalonWorldGenerator
 from avalon.datagen.world_creation.world_generator import BlockingWorldGenerator
@@ -165,7 +166,7 @@ def task_groups_from_training_protocol(
 @attr.s(auto_attribs=True, frozen=True)
 class GodotEnvironmentParams(EnvironmentParams):
     suite: str = "godot"
-    seed: int = 0
+    # seed: int = 0
     # whether to use per-task curriculum learning. requires a wrapper.
     is_task_curriculum_used: bool = True
     # how much to update the task difficulty per successful or failed episode.
@@ -222,6 +223,16 @@ class GodotEnvironmentParams(EnvironmentParams):
     # difficulty bin size for eval logging histograms
     eval_difficulty_bin_size: float = 0.1
 
+    info_fields: list[str] = [
+        "cumulative_episode_return",
+        "cumulative_episode_length",
+        "task",
+        "difficulty",
+        "success",
+        "score",
+        "world_index",
+    ]
+
     @property
     def is_fixed_generator(self) -> bool:
         """Use the `fixed` world genenerator for evaluation."""
@@ -245,8 +256,12 @@ class GodotEnvironmentParams(EnvironmentParams):
     def num_tasks(self) -> int:
         return len(get_all_tasks_for_task_groups(self.task_groups))
 
+    @property
+    def seed(self) -> int:
+        return self.env_index
 
-def write_video_from_np_arrays(video_path: Path, arrays: List[np.ndarray], fps: float = 20):
+
+def write_video_from_np_arrays(video_path: Path, arrays: List[np.ndarray], fps: float = 20) -> None:
     # size = arrays[0].shape[:2]
     # out = cv2.VideoWriter(str(video_path), cv2.VideoWriter_fourcc(*"mp4v"), fps, size, False)
     # for array in arrays:
@@ -258,7 +273,7 @@ def write_video_from_np_arrays(video_path: Path, arrays: List[np.ndarray], fps:
 
 
 class AvalonEnv(GodotEnv[AvalonObservation, VRAction, GeneratedAvalonWorldParams]):
-    def __init__(self, params: GodotEnvironmentParams):
+    def __init__(self, params: GodotEnvironmentParams) -> None:
         self.params = params
 
         base_config = create_base_benchmark_config(
@@ -346,7 +361,7 @@ class AvalonEnv(GodotEnv[AvalonObservation, VRAction, GeneratedAvalonWorldParams
 
         return observation, goal_progress
 
-    def update_lame_observation(self, lame_observation: Dict[str, Any]):
+    def update_lame_observation(self, lame_observation: Dict[str, Any]) -> None:
         if self.params.is_frame_id_transformed_to_frames_remaining and "frame_id" in lame_observation:
             assert isinstance(self.goal_evaluator, AvalonGoalEvaluator)
             lame_observation["frame_id"] = self.goal_evaluator.frame_limit - lame_observation["frame_id"] - 1
@@ -434,7 +449,7 @@ class AvalonEnv(GodotEnv[AvalonObservation, VRAction, GeneratedAvalonWorldParams
         state["_bridge"] = None
         return state
 
-    def __setstate__(self, state: Dict[str, Any]):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         """See __getstate__ docstring
 
         Args:
@@ -459,12 +474,12 @@ class AvalonEnv(GodotEnv[AvalonObservation, VRAction, GeneratedAvalonWorldParams
         else:
             assert False
 
-    def set_task_difficulty(self, task: AvalonTask, difficulty: float):
+    def set_task_difficulty(self, task: AvalonTask, difficulty: float) -> None:
         assert not self.params.is_fixed_generator
         assert isinstance(self.world_generator, (LocalProcessWorldGenerator, BlockingWorldGenerator))
         self.world_generator.set_task_difficulty(task, difficulty)
 
-    def set_meta_difficulty(self, difficulty: float):
+    def set_meta_difficulty(self, difficulty: float) -> None:
         assert not self.params.is_fixed_generator
         assert isinstance(self.world_generator, (LocalProcessWorldGenerator, BlockingWorldGenerator))
         self.world_generator.set_meta_difficulty(difficulty)
@@ -508,7 +523,7 @@ def _transform_observation_clamp(x: NDArray, min: Optional[float] = None, max: O
 
 
 class GodotObsTransformWrapper(gym.ObservationWrapper):
-    def __init__(self, env: GodotEnv, greyscale: bool = False):
+    def __init__(self, env: GodotEnv, greyscale: bool = False) -> None:
         super().__init__(env)
         self.transforms = {
             "rgbd": partial(_transform_observation_rgbd, greyscale=greyscale),
@@ -577,7 +592,7 @@ def squash_real(x: np.ndarray) -> np.ndarray:
 class ScaleAndSquashAction(gym.ActionWrapper):
     """Constrain to within [-1, 1] with a tanh and then scale the actions."""
 
-    def __init__(self, env: gym.Env, key: str = "real", scale: float = 1):
+    def __init__(self, env: gym.Env, key: str = "real", scale: float = 1) -> None:
         super().__init__(env)
         self.key = key
         self.scale = scale
@@ -593,7 +608,7 @@ class ScaleAndSquashAction(gym.ActionWrapper):
 
 # Note: the rate of update of difficulty (per global env step) depends on the number of workers.
 class CurriculumWrapper(Wrapper):
-    def __init__(self, env: AvalonEnv, task_difficulty_update: float, meta_difficulty_update: float):
+    def __init__(self, env: AvalonEnv, task_difficulty_update: float, meta_difficulty_update: float) -> None:
         super().__init__(env)
         self._env = env
         self.difficulties: DefaultDict[AvalonTask, float] = defaultdict(float)
@@ -604,7 +619,7 @@ class CurriculumWrapper(Wrapper):
     def step(self, action: Dict[str, torch.Tensor]):
         observation, reward, done, info = self.env.step(action)  # type: ignore[misc]
         if done:
-            task = AvalonTask[info["task"]]
+            task = AvalonTask[int_to_avalon_task[int(info["task"])]]
             update_step = self.task_difficulty_update  # * np.random.uniform()
             if info["success"] == 1:
                 self.difficulties[task] += update_step
