@@ -23,18 +23,20 @@ from uuid import uuid4
 import numpy as np
 import wandb
 
+from avalon.agent.evaluation import EVAL_TEMP_PATH
 from avalon.agent.godot.godot_gym import CURRICULUM_BASE_PATH
-from avalon.agent.random.evaluation import EVAL_TEMP_PATH
 from avalon.agent.torchbeast import polybeast_env
 from avalon.agent.torchbeast import polybeast_learner
 from avalon.agent.torchbeast.avalon_helpers import TORCHBEAST_ENV_LOGS_PATH
 from avalon.agent.torchbeast.avalon_helpers import _destroy_process_tree
+from avalon.common.log_utils import configure_parent_logging
 from avalon.common.log_utils import logger
 from avalon.contrib.s3_utils import SimpleS3Client
 from avalon.contrib.utils import FILESYSTEM_ROOT
 
 
-def run_env(flags, actor_id, exit_event):
+def run_env(flags, actor_id, exit_event) -> None:
+    configure_parent_logging()
     np.random.seed()  # Get new random seed in forked process.
     polybeast_env.main(flags, exit_event)
 
@@ -57,7 +59,7 @@ def get_flags(check_argv_empty: bool = False):
     return flags
 
 
-def restore_curriculum_files():
+def restore_curriculum_files() -> None:
     api = wandb.Api()
     run_api = api.run(wandb.run.path)
     curriculum_filenames = [file.name for file in run_api.files() if file.name.startswith("curriculum")]
