@@ -30,7 +30,6 @@ from typing import Set
 import libtorchbeast
 import nest
 import numpy as np
-import sentry_sdk
 import torch
 import wandb
 from gym import spaces
@@ -55,6 +54,7 @@ from avalon.agent.torchbeast.avalon_helpers import unflatten_tensor
 from avalon.agent.torchbeast.avalon_helpers import vtrace_from_dist
 from avalon.agent.torchbeast.core.environment import Environment
 from avalon.agent.torchbeast.model import DictActionHead
+from avalon.common.error_utils import setup_sentry
 from avalon.common.log_utils import logger
 from avalon.common.visual_utils import encode_video
 from avalon.common.wandb_utils import get_latest_checkpoint_filename
@@ -148,7 +148,7 @@ parser.add_argument("--save_test_actions", action="store_true",
 # yapf: enable
 
 
-def _set_seed(seed):
+def _set_seed(seed) -> None:
     torch.manual_seed(seed)
     random.seed(seed)
     np.random.seed(seed)
@@ -183,7 +183,7 @@ class Net(nn.Module):
         img_dim: int = 96,
         encoder_out_dim: int = 256,
         is_sampling_mode: bool = False,
-    ):
+    ) -> None:
         super(Net, self).__init__()
         # TODO: get real action space
 
@@ -271,7 +271,7 @@ class Net(nn.Module):
         return unflatten_tensor(self._action_space, flat_action)
 
 
-def inference(flags, inference_batcher, model, lock=threading.Lock()):  # noqa: B008
+def inference(flags, inference_batcher, model, lock=threading.Lock()) -> None:  # noqa: B008
     with torch.no_grad():
         for batch in inference_batcher:
             batched_env_outputs, agent_state = batch.get_inputs()
@@ -300,7 +300,7 @@ def learn(
     scheduler,
     stats,
     lock=threading.Lock(),
-):
+) -> None:
     model.train()
     for tensors in learner_queue:
         if flags.mode != "train":
@@ -575,7 +575,7 @@ def train(flags):
     for t in learner_threads + inference_threads:
         t.start()
 
-    def checkpoint(step):
+    def checkpoint(step) -> None:
         if flags.disable_checkpoint:
             return
         checkpointpath = Path(wandb.run.dir) / f"model_step_{step}.tar"
@@ -683,13 +683,7 @@ def main(flags):
     if not flags.pipes_basename.startswith("unix:"):
         raise Exception("--pipes_basename has to be of the form unix:/some/path.")
 
-    sentry_sdk.init(
-        dsn="https://198a62315b2c4c2a99cb8a5493224e2f@o568344.ingest.sentry.io/6453090",
-        # Set traces_sample_rate to 1.0 to capture 100%
-        # of transactions for performance monitoring.
-        # We recommend adjusting this value in production.
-        traces_sample_rate=1.0,
-    )
+    setup_sentry(percent_of_traces_to_capture=1.0)
 
     os.makedirs(GODOT_ERROR_LOG_PATH, exist_ok=True)
     wandb.save(
