@@ -1,10 +1,9 @@
-import warnings
-
 import attr
 
 from avalon.agent.common.parse_args import parse_args
 from avalon.agent.common.test_envs import TestEnvironmentParams
 from avalon.agent.common.trainer import OffPolicyTrainer
+from avalon.agent.common.util import setup_new_process
 from avalon.agent.dreamer.params import DreamerParams
 from avalon.common.log_utils import configure_remote_logger
 
@@ -42,18 +41,19 @@ class DreamerTestEnvParams(DreamerParams):
     log_freq_media: int = 25
 
 
-def run(params: DreamerParams):
+def run(params: DreamerParams) -> None:
     trainer = OffPolicyTrainer(params)
     try:
         trainer.start_train_rollouts()
         trainer.train()
     finally:
-        trainer.shutdown()
+        trainer.shutdown(finish_wandb_quietly=True)
+        del trainer.train_dataloader
 
 
 if __name__ == "__main__":
-    warnings.filterwarnings("ignore", category=DeprecationWarning)
-    configure_remote_logger()
+    configure_remote_logger(level="INFO")
+    setup_new_process()
     default_params = DreamerTestEnvParams()
     default_params = parse_args(default_params)
     run(default_params)
