@@ -1,11 +1,12 @@
-import warnings
-
 import attr
+from loguru import logger
 
 from avalon.agent.common.parse_args import parse_args
 from avalon.agent.common.test_envs import TestEnvironmentParams
 from avalon.agent.common.trainer import OnPolicyTrainer
+from avalon.agent.common.util import setup_new_process
 from avalon.agent.ppo.params import PPOParams
+from avalon.common.log_utils import configure_remote_logger
 
 
 @attr.s(auto_attribs=True, frozen=True)
@@ -21,8 +22,10 @@ class TestPPOParams(PPOParams):
     log_freq_media: int = 25
     entropy_coef: float = 1e-3  # with a small entropy penalty, random actions should stay random unless useful
 
+    __test__ = False
 
-def run(params: TestPPOParams):
+
+def run(params: TestPPOParams) -> None:
     assert params.num_batches == 1
     assert params.batch_size == params.num_steps * params.num_workers
 
@@ -34,7 +37,12 @@ def run(params: TestPPOParams):
 
 
 if __name__ == "__main__":
-    warnings.filterwarnings("ignore", category=DeprecationWarning)
-    default_params = TestPPOParams()
-    default_params = parse_args(default_params)
-    run(default_params)
+    configure_remote_logger(level="INFO")
+    setup_new_process()
+    try:
+        default_params = TestPPOParams()
+        default_params = parse_args(default_params)
+        run(default_params)
+    except Exception as e:
+        logger.exception(e)
+        raise
