@@ -12,5 +12,5 @@ fi
 mypy_targets="$1"
 
 #actually run mypy
-(MYPYPATH=.:../avalon:../../computronium:../../science:../../bones mypy --show-absolute-path --config-file ./mypy.ini $mypy_targets | grep -v 'Function is missing a return type annotation' | grep -v 'Use \"-> None\" if function does not return a value' | grep -E "(error:|note:)" | cat ) || echo "Done."
+(MYPYPATH=.:../avalon:../../computronium:../../science:../../bones PYTHONPATH=$PYTHONPATH:../../formatters/python mypy --show-absolute-path --config-file ./mypy.ini $mypy_targets | grep -v 'Function is missing a return type annotation' | grep -v 'Use \"-> None\" if function does not return a value' | grep -E "(error:|note:)" | cat ) || echo "Done."
 # (MYPYPATH=.:../avalon:../../computronium:../../science:../../bones dmypy run -- --show-absolute-path --config-file ./common/mypy.ini $mypy_targets | grep -v 'Function is missing a return type annotation' | grep -v 'Use \"-> None\" if function does not return a value' | grep -E "(error:|note:)" | cat ) || echo "Done."
