@@ -3,7 +3,7 @@ import sys
 from fire import helptext
 
 
-def make_fire_more_friendly(line_length: int = 200):
+def make_fire_more_friendly(line_length: int = 200) -> None:
     # make fire stop truncating so many things
     helptext.LINE_LENGTH = line_length
 
