@@ -29,6 +29,7 @@ from godot_parser import Node as GDNode
 from typing_extensions import TypeGuard
 
 from avalon.common.errors import SwitchError
+from avalon.common.log_utils import configure_parent_logging
 from avalon.common.log_utils import logger
 from avalon.contrib.serialization import Serializable
 from avalon.datagen.errors import ImpossibleWorldError
@@ -108,7 +109,7 @@ class GeneratedAvalonWorldParams(GenerateAvalonWorldParams, GeneratedWorldParams
         with open(existing_world_path / GeneratedAvalonWorldParams.SERIALIZED_FILE_NAME) as file:
             return GeneratedAvalonWorldParams.from_dict(json.load(file))
 
-    def save_to_output_path(self):
+    def save_to_output_path(self) -> None:
         serialized_path = self.output_path / self.SERIALIZED_FILE_NAME
         with open(serialized_path, "w") as file:
             json.dump(self.to_dict(), file)
@@ -119,7 +120,7 @@ GeneratedWorldParamsType = TypeVar("GeneratedWorldParamsType", bound=GeneratedWo
 
 
 class WorldGenerator(Generic[GeneratedWorldParamsType]):
-    def __init__(self, base_path: Path, seed: int):
+    def __init__(self, base_path: Path, seed: int) -> None:
         self.output_path = base_path / str(uuid.uuid4())
         self.output_path.mkdir(parents=True, exist_ok=False)
         self.set_seed(seed)
@@ -131,16 +132,16 @@ class WorldGenerator(Generic[GeneratedWorldParamsType]):
         """
         raise NotImplementedError()
 
-    def set_seed(self, seed: int):
+    def set_seed(self, seed: int) -> None:
         self._seed = seed
 
-    def close(self):
+    def close(self) -> None:
         if self.output_path.exists():
             shutil.rmtree(self.output_path)
 
 
 class EmptyLevelGenerator(WorldGenerator[GeneratedWorldParams]):
-    def __init__(self, base_path: Path, seed: int):
+    def __init__(self, base_path: Path, seed: int) -> None:
         super().__init__(base_path, seed)
         scene = GodotScene()
         with scene.use_tree() as tree:
@@ -202,7 +203,7 @@ def is_fixed_world_generator(
 
 
 class SingleTaskWorldGenerator(AvalonWorldGenerator):
-    def __init__(self, base_path: Path, seed: int, difficulty: float, task: AvalonTask):
+    def __init__(self, base_path: Path, seed: int, difficulty: float, task: AvalonTask) -> None:
         super().__init__(base_path, seed)
         self.difficulty = difficulty
         self.task = task
@@ -248,7 +249,7 @@ class BlockingWorldGenerator(AvalonWorldGenerator):
         seed: int,
         start_difficulty: float,
         task_groups: Tuple[AvalonTaskGroup, ...],
-    ):
+    ) -> None:
         super().__init__(base_path, seed)
         self.difficulties = {x: start_difficulty for x in get_all_tasks_for_task_groups(task_groups)}
         self.task_groups = task_groups
@@ -278,17 +279,17 @@ class BlockingWorldGenerator(AvalonWorldGenerator):
         self.task = task.task
         self.difficulties[task.task] = task.difficulty
 
-    def set_task_difficulty(self, task: AvalonTask, difficulty: float):
+    def set_task_difficulty(self, task: AvalonTask, difficulty: float) -> None:
         self.difficulties[task] = difficulty
 
-    def set_meta_difficulty(self, difficulty: float):
+    def set_meta_difficulty(self, difficulty: float) -> None:
         self.meta_difficulty = difficulty
 
     def get_task(self) -> WorldGeneratorTask:
         assert self.task is not None
         return WorldGeneratorTask(task=self.task, difficulty=self.difficulties[self.task])
 
-    def set_seed(self, seed: int):
+    def set_seed(self, seed: int) -> None:
         super().set_seed(seed)
 
 
@@ -325,7 +326,7 @@ def generate_fixed_worlds(
 
     ctx = mp.get_context("spawn")
 
-    def on_done(result: GeneratedAvalonWorldParams):
+    def on_done(result: GeneratedAvalonWorldParams) -> None:
         # logger.info(f"Finished generating {result}")
         worlds[result.index] = result
 
@@ -333,7 +334,9 @@ def generate_fixed_worlds(
         logger.error(f"World generation failed! {error}")
         raise error
 
-    with ctx.Pool(processes=min(len(world_params), num_processes)) as worker_pool:
+    with ctx.Pool(
+        processes=min(len(world_params), num_processes), initializer=configure_parent_logging
+    ) as worker_pool:
         for params in world_params:
             worker_pool.apply_async(
                 generate_world,
@@ -369,7 +372,7 @@ class FixedWorldLoader(AvalonWorldGenerator):
         generated_worlds_path: Path,
         num_generators: int = 1,
         generator_index: int = 0,
-    ):
+    ) -> None:
         unused_placeholder_seed = -1
         super().__init__(base_path, seed=unused_placeholder_seed)
         self._init_by_loading(generated_worlds_path, num_generators, generator_index)
@@ -421,7 +424,7 @@ class FixedWorldGenerator(AvalonWorldGenerator):
         task_groups: Tuple[AvalonTaskGroup, ...],
         num_generators: int = 1,
         generator_index: int = 0,
-    ):
+    ) -> None:
         super().__init__(base_path, seed)
         self._init_by_generating(difficulties, task_groups, num_generators, generator_index)
 
@@ -431,7 +434,7 @@ class FixedWorldGenerator(AvalonWorldGenerator):
         task_groups: Tuple[AvalonTaskGroup, ...],
         num_generators: int,
         generator_index: int,
-    ):
+    ) -> None:
         world_params = get_world_params_for_task_groups(
             task_groups,
             difficulties,
@@ -451,9 +454,10 @@ class FixedWorldGenerator(AvalonWorldGenerator):
         return [_copy_world(self.worlds[start_world_id])]
 
 
-def disable_sigint():
+def disable_sigint() -> None:
     # Disable sigint handler so we can handle cleanup neatly ourselves.
     signal.signal(signal.SIGINT, signal.SIG_IGN)
+    configure_parent_logging()
 
 
 # TODO: this introduces some non-determinism, fix it by anchoring to ids of levels generated
@@ -467,9 +471,8 @@ class LocalProcessWorldGenerator(AvalonWorldGenerator):
         start_difficulty: float = 0,
         num_workers: int = 2,
         buffer_size: int = 20,
-        offset: int = 0,
         is_task_curriculum_used: bool = True,
-    ):
+    ) -> None:
         super().__init__(base_path, seed)
         self.task_groups = task_groups
         self.min_difficulty = min_difficulty
@@ -480,7 +483,7 @@ class LocalProcessWorldGenerator(AvalonWorldGenerator):
         self.buffer: List[GeneratedAvalonWorldParams] = []
         self.lock = Lock()
         self.worker_pool = Pool(processes=num_workers, initializer=disable_sigint)
-        self.offset = offset
+        self.offset = 0  # roughly, stores the index of the world to be generated next.
         self._request_batch(buffer_size)
 
     def generate_batch(self, start_world_id: Optional[int], batch_size: int = 1) -> List[GeneratedAvalonWorldParams]:
@@ -493,19 +496,19 @@ class LocalProcessWorldGenerator(AvalonWorldGenerator):
             except InsufficientBufferSize:
                 time.sleep(0.1)
 
-    def close(self):
+    def close(self) -> None:
         if self.worker_pool is not None:
             self._destroy_pool()
         super().close()
 
-    def _destroy_pool(self):
+    def _destroy_pool(self) -> None:
         self.worker_pool.close()
         # These are pretty quick, just let them complete.
         # self.worker_pool.terminate()
         self.worker_pool.join()
-        self.worker_pool = None
+        self.worker_pool = None  # type: ignore
 
-    def _request_batch(self, batch_size: int):
+    def _request_batch(self, batch_size: int) -> None:
         with self.lock:
             sampled_tasks = [
                 rand_task_with_difficulty(self.task_groups, self.meta_difficulty, self.rand) for _ in range(batch_size)
@@ -545,23 +548,23 @@ class LocalProcessWorldGenerator(AvalonWorldGenerator):
                 return result
             raise InsufficientBufferSize()
 
-    def _on_world_generation_done(self, result: GeneratedAvalonWorldParams):
+    def _on_world_generation_done(self, result: GeneratedAvalonWorldParams) -> None:
         with self.lock:
             self.buffer.append(result)
 
-    def _on_world_generation_error(self, error: BaseException):
+    def _on_world_generation_error(self, error: BaseException) -> None:
         logger.error("Caught error in _on_world_generation_error. This shouldn't happen!!!:")
         logger.error(error)
 
-    def set_task_difficulty(self, task: AvalonTask, difficulty: float):
+    def set_task_difficulty(self, task: AvalonTask, difficulty: float) -> None:
         with self.lock:
             self.difficulties[task] = max(self.min_difficulty, difficulty)
 
-    def set_meta_difficulty(self, difficulty: float):
+    def set_meta_difficulty(self, difficulty: float) -> None:
         with self.lock:
             self.meta_difficulty = difficulty
 
-    def set_seed(self, seed: int):
+    def set_seed(self, seed: int) -> None:
         super().set_seed(seed)
         # TODO: can this be a np.random.Generator as well?
         self.rand = Random(f"seed:{self._seed}")  # type: ignore
@@ -605,6 +608,7 @@ GENERATION_FUNCTION_BY_TASK: Final[Dict[AvalonTask, TaskGenerationFunction]] = {
 MAX_NOISE_SCALE = 0.3
 
 
+@logger.catch(reraise=True)
 def generate_world(params: GenerateAvalonWorldParams) -> GeneratedAvalonWorldParams:
     warnings.filterwarnings("ignore", category=DeprecationWarning)
     output_path = Path(params.output)
